@@ -1,4 +1,4 @@
-#!/usr/bin/perl -w
+#!/usr/bin/perl
 
 # debi:  Install current version of deb package
 # debc:  List contents of current version of deb package
@@ -21,16 +21,17 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program. If not, see <http://www.gnu.org/licenses/>.
+# along with this program. If not, see <https://www.gnu.org/licenses/>.
 
 use 5.008;
 use strict;
-use Getopt::Long qw(:config gnu_getopt);
+use warnings;
+use Getopt::Long qw(:config bundling permute no_getopt_compat);
 use File::Basename;
 use filetest 'access';
 use Cwd;
 use Dpkg::Control;
-use Dpkg::Changelog::Parse;
+use Dpkg::Changelog::Parse qw(changelog_parse);
 
 my $progname = basename($0,'.pl');  # the '.pl' is for when we're debugging
 my $modified_conf_msg;
@@ -85,6 +86,8 @@ Usage: $progname [options] [.changes file] [package ...]
     -t<target>        Search for changes file made for GNU <target> arch
     --debs-dir DIR    Look for the changes and debs files in DIR instead of
                       the parent of the current package directory
+    --list-changes    only list the .changes file
+    --list-debs       only list the .deb files; don't display their contents
     --multi           Search for multiarch .changes file made by dpkg-cross
     --check-dirname-level N
                       How much to check directory names:
@@ -119,7 +122,7 @@ the GNU General Public License, version 2 or later.
 EOF
 
 # Start by setting default values
-my $debsdir = '..';
+my $debsdir;
 my $debsdir_warning;
 my $check_dirname_level = 1;
 my $check_dirname_regex = 'PACKAGE(-.+)?';
@@ -159,9 +162,7 @@ if (@ARGV and $ARGV[0] =~ /^--no-?conf$/) {
     # installing a broken package
     $config_vars{'DEBRELEASE_DEBS_DIR'} =~ s%/+%/%;
     $config_vars{'DEBRELEASE_DEBS_DIR'} =~ s%(.)/$%$1%;
-    if (! -d $config_vars{'DEBRELEASE_DEBS_DIR'}) {
-	$debsdir_warning = "config file specified DEBRELEASE_DEBS_DIR directory $config_vars{'DEBRELEASE_DEBS_DIR'} does not exist!";
-    }
+    $debsdir_warning = "config file specified DEBRELEASE_DEBS_DIR directory $config_vars{'DEBRELEASE_DEBS_DIR'} does not exist!";
 
     foreach my $var (sort keys %config_vars) {
 	if ($config_vars{$var} ne $config_default{$var}) {
@@ -181,6 +182,7 @@ my ($opt_help, $opt_version, $opt_a, $opt_t, $opt_debsdir, $opt_multi);
 my $opt_upgrade;
 my ($opt_level, $opt_regex, $opt_noconf);
 my ($opt_tool, $opt_with_depends);
+my ($opt_list_changes, $opt_list_debs);
 GetOptions("help" => \$opt_help,
 	   "version" => \$opt_version,
 	   "a=s" => \$opt_a,
@@ -194,6 +196,8 @@ GetOptions("help" => \$opt_help,
 	   "tool=s" => \$opt_tool,
 	   "noconf" => \$opt_noconf,
 	   "no-conf" => \$opt_noconf,
+	   "list-changes" => \$opt_list_changes,
+	   "list-debs" => \$opt_list_debs,
 	   )
     or die "Usage: $progname [options] [.changes file] [package ...]\nRun $progname --help for more details\n";
 
@@ -207,19 +211,6 @@ my ($targetarch, $targetgnusystem);
 $targetarch = $opt_a ? "-a$opt_a" : "";
 $targetgnusystem = $opt_t ? "-t$opt_t" : "";
 
-if ($opt_debsdir) {
-    $opt_debsdir =~ s%/+%/%;
-    $opt_debsdir =~ s%(.)/$%$1%;
-    if (! -d $opt_debsdir) {
-	$debsdir_warning = "--debs-dir directory $opt_debsdir does not exist!";
-    }
-    $debsdir = $opt_debsdir;
-}
-
-if ($debsdir_warning) {
-    die "$progname: $debsdir_warning\n";
-}
-
 if (defined $opt_level) {
     if ($opt_level =~ /^[012]$/) { $check_dirname_level = $opt_level; }
     else {
@@ -249,6 +240,17 @@ chomp $arch;
 my $chdir = 0;
 
 if (! defined $changes) {
+    if ($opt_debsdir) {
+	$opt_debsdir =~ s%/+%/%;
+	$opt_debsdir =~ s%(.)/$%$1%;
+	$debsdir_warning = "--debs-dir directory $opt_debsdir does not exist!";
+	$debsdir = $opt_debsdir;
+    }
+
+    if (! -d $debsdir) {
+	die "$progname: $debsdir_warning\n";
+    }
+
     # Look for .changes file via debian/changelog
     until (-r 'debian/changelog') {
 	$chdir = 1;
@@ -323,6 +325,11 @@ EOF
     }
 }
 
+if ($opt_list_changes) {
+  printf "%s\n", $changes;
+  exit(0);
+}
+
 chdir dirname($changes)
     or die "$progname: can't chdir to $changes directory: $!\n";
 $changes = basename($changes);
@@ -388,6 +395,10 @@ if ($progname eq 'debi') {
 } else {
     # $progname eq 'debc'
     foreach my $deb (@debs) {
+	if ($opt_list_debs) {
+	    printf "%s/%s\n", cwd(), $deb;
+	    next;
+	}
 	print "$deb\n";
 	print '-' x length($deb), "\n";
 	system('dpkg-deb', '-I', $deb) == 0
